#include "stdafx.h"
#include "UICarBodyWnd.h"
#include "UITradeWnd.h"
#include "xrXMLParser.h"
#include "UIXmlInit.h"
#include "../Entity.h"
#include "../HUDManager.h"
#include "../WeaponAmmo.h"
#include "../Actor.h"
#ifdef DEBUG
#include "../ode_include.h"
#include "../../StatGraph.h"
#include "../PHDebug.h"
#endif
#include "../hit.h"
#include "../PHDestroyable.h"
#include "../Car.h"
#include "../Trade.h"
#include "../UIGameSP.h"
#include "../eatable_item.h"
#include "../inventory.h"
#include "UIInventoryUtilities.h"
#include "../level.h"
#include "../profiler.h"
#include "UIWpnDragDropItem.h"
#include "../ai/monsters/BaseMonster/base_monster.h"
#include "../WeaponMagazined.h"

using namespace InventoryUtilities;

#define				CAR_BODY_XML		"carbody_new.xml"
#define				CARBODY_ITEM_XML	"carbody_item.xml"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CUICarBodyWnd::CUICarBodyWnd()
{
	Init();
	Hide();

	SetFont(HUD().Font().pFontMedium);
	m_b_need_update		= false;
}

//////////////////////////////////////////////////////////////////////////

CUICarBodyWnd::~CUICarBodyWnd()
{
	ClearDragDrop(m_vDragDropItems);
}

//////////////////////////////////////////////////////////////////////////

void CUICarBodyWnd::Init()
{
	CUIXml uiXml;
	uiXml.Init(CONFIG_PATH, UI_PATH, CAR_BODY_XML);
	
	CUIXmlInit xml_init;

	CUIWindow::Init(CUIXmlInit::ApplyAlignX(0, alCenter),
					CUIXmlInit::ApplyAlignY(0, alCenter),
					UI_BASE_WIDTH, UI_BASE_HEIGHT);

	//  
	AttachChild(&UIStaticTop);
	UIStaticTop.Init("ui\\ui_top_background", 0,0,UI_BASE_WIDTH,128);
	AttachChild(&UIStaticBottom);
	UIStaticBottom.Init("ui\\ui_bottom_background", 0,UI_BASE_HEIGHT-32,UI_BASE_WIDTH,32);

	//       
	AttachChild(&UIOurIcon);
	xml_init.InitStatic(uiXml, "static_icon", 0, &UIOurIcon);
	AttachChild(&UIOthersIcon);
	xml_init.InitStatic(uiXml, "static_icon", 1, &UIOthersIcon);
	UIOurIcon.AttachChild(&UICharacterInfoLeft);
	UICharacterInfoLeft.Init(0,0, UIOurIcon.GetWidth(), UIOurIcon.GetHeight(), "trade_character.xml");
	UIOthersIcon.AttachChild(&UICharacterInfoRight);
	UICharacterInfoRight.Init(0,0, UIOthersIcon.GetWidth(), UIOthersIcon.GetHeight(), "trade_character.xml");

	// 
	AttachChild(&UIOurBagWnd);
	xml_init.InitStatic(uiXml, "our_bag_static", 0, &UIOurBagWnd);
	AttachChild(&UIOthersBagWnd);
	xml_init.InitStatic(uiXml, "others_bag_static", 0, &UIOthersBagWnd);

	// Drag&Drop
	UIOurBagWnd.AttachChild(&UIOurBagList);	
	xml_init.InitDragDropList(uiXml, "dragdrop_list", 0, &UIOurBagList);
	UIOurBagList.SetItemsScaleXY(TRADE_ICONS_SCALE, TRADE_ICONS_SCALE);

	UIOthersBagWnd.AttachChild(&UIOthersBagList);	
	xml_init.InitDragDropList(uiXml, "dragdrop_list", 1, &UIOthersBagList);
	UIOthersBagList.SetItemsScaleXY(TRADE_ICONS_SCALE, TRADE_ICONS_SCALE);


	//  
	AttachChild(&UIDescWnd);
	xml_init.InitFrameWindow(uiXml, "frame_window", 0, &UIDescWnd);
	UIDescWnd.AttachChild(&UIStaticDesc);
	xml_init.InitStatic(uiXml, "descr_static", 0, &UIStaticDesc);
	UIDescWnd.AttachChild(&UIItemInfo);
	UIItemInfo.Init(0,0, UIDescWnd.GetWidth(), UIDescWnd.GetHeight(), CARBODY_ITEM_XML);


	//  
	xml_init.InitAutoStatic(uiXml, "auto_static", this);

	AttachChild(&UIPropertiesBox);
	UIPropertiesBox.Init("ui_pop_up",0,0,300,300);
	UIPropertiesBox.Hide();

	UIOurBagList.SetCheckProc(OurBagProc);
	UIOthersBagList.SetCheckProc(OthersBagProc);

	SetCurrentItem(NULL);
	m_pCurrentDragDropItem = NULL;
	UIStaticDesc.SetText(NULL);

	AttachChild(&UITakeAll);
	xml_init.InitButton(uiXml, "take_all_btn", 0, &UITakeAll);
}

//////////////////////////////////////////////////////////////////////////

void CUICarBodyWnd::InitCarBody(CInventory* pOurInv,    CGameObject* pOurObject,
								CInventory* pOthersInv, CGameObject* pOthersObject)
{

    m_pOurObject			= pOurObject;
	m_pOthersObject			= pOthersObject;

	m_pInv					= pOurInv;
	m_pOthersInv			= pOthersInv;

	
//	CInventoryOwner* pOurInvOwner = smart_cast<CInventoryOwner*>(pOurObject);
	UICharacterInfoLeft.InitCharacter(pOurObject->ID());
//	CInventoryOwner* pOthersInvOwner = smart_cast<CInventoryOwner*>(pOthersObject);

	if(pOthersObject) {
		CBaseMonster *monster = smart_cast<CBaseMonster *>(pOthersObject);
		if (!monster) {
			UICharacterInfoRight.InitCharacter(pOthersObject->ID());
		} else {
			UICharacterInfoRight.ClearInfo();
		}
	}

	m_pMouseCapturer			= NULL;
	UIPropertiesBox.Hide		();
//	UIMessageBox.Hide();
	EnableAll					();


	UpdateLists					();

}  
void CUICarBodyWnd::UpdateLists_delayed()
{
		m_b_need_update = true;
}

void CUICarBodyWnd::UpdateLists()
{
//	CTimer	T;
//	T.Start();
	//   
	UIOurBagList.DropAll();
	UIOthersBagList.DropAll();

	ClearDragDrop(m_vDragDropItems);

	ruck_list.clear	();
	m_pInv->AddAvailableItems(ruck_list);
	std::sort		(ruck_list.begin(),ruck_list.end(),GreaterRoomInRuck);

	// 
	TIItemContainer::iterator it;
	for(it =  ruck_list.begin(); ruck_list.end() != it; ++it) 
	{
		if((*it)) 
		{
			m_vDragDropItems.push_back(xr_new<CUIWpnDragDropItem>());
	
			CUIDragDropItem& UIDragDropItem = *m_vDragDropItems.back();


			UIDragDropItem.CUIStatic::Init(0, 0, INV_GRID_WIDTH, INV_GRID_HEIGHT);
			UIDragDropItem.SetShader(GetEquipmentIconsShader());

			UIDragDropItem.SetGridHeight((*it)->GetGridHeight());
			UIDragDropItem.SetGridWidth((*it)->GetGridWidth());

			UIDragDropItem.GetUIStaticItem().SetOriginalRect(
									float((*it)->GetXPos()*INV_GRID_WIDTH),
									float((*it)->GetYPos()*INV_GRID_HEIGHT),
									float((*it)->GetGridWidth()*INV_GRID_WIDTH),
									float((*it)->GetGridHeight()*INV_GRID_HEIGHT));

			UIDragDropItem.SetData((*it));

			UIDragDropItem.SetFont(HUD().Font().pFontLetterica16Russian);

			CWeaponAmmo* pWeaponAmmo  = smart_cast<CWeaponAmmo*>((*it));
			if(pWeaponAmmo)	UIDragDropItem.SetCustomDraw(AmmoDrawProc);

			CEatableItem* pEatableItem = smart_cast<CEatableItem*>((*it));
			if(pEatableItem) UIDragDropItem.SetCustomDraw(FoodDrawProc);


			UIDragDropItem.SetStretchTexture(true);
				
			UIOurBagList.AttachChild(&UIDragDropItem);
		}
	}


	ruck_list.clear	();
	m_pOthersInv->AddAvailableItems(ruck_list);
	std::sort		(ruck_list.begin(),ruck_list.end(),GreaterRoomInRuck);

	// 
	for(it =  ruck_list.begin(); ruck_list.end() != it; ++it) 
	{
		if((*it)) 
		{
			m_vDragDropItems.push_back(xr_new<CUIWpnDragDropItem>());
		
			CUIDragDropItem& UIDragDropItem = *m_vDragDropItems.back();

				
			UIDragDropItem.CUIStatic::Init(0, 0, INV_GRID_WIDTH, INV_GRID_HEIGHT);

			UIDragDropItem.SetShader(GetEquipmentIconsShader());

			UIDragDropItem.SetGridHeight((*it)->GetGridHeight());
			UIDragDropItem.SetGridWidth((*it)->GetGridWidth());

			UIDragDropItem.GetUIStaticItem().SetOriginalRect(
									float((*it)->GetXPos()*INV_GRID_WIDTH),
									float((*it)->GetYPos()*INV_GRID_HEIGHT),
									float((*it)->GetGridWidth()*INV_GRID_WIDTH),
									float((*it)->GetGridHeight()*INV_GRID_HEIGHT));
			UIDragDropItem.SetData((*it));

			UIDragDropItem.SetFont(HUD().Font().pFontLetterica16Russian);

			CWeaponAmmo* pWeaponAmmo  = smart_cast<CWeaponAmmo*>((*it));
			if(pWeaponAmmo)	UIDragDropItem.SetCustomDraw(AmmoDrawProc);

			CEatableItem* pEatableItem = smart_cast<CEatableItem*>((*it));
			if(pEatableItem) UIDragDropItem.SetCustomDraw(FoodDrawProc);

			UIDragDropItem.SetStretchTexture(true);

			UIOthersBagList.AttachChild(&UIDragDropItem);
		}
	}

	UpdateWeight(UIOurBagWnd);
	m_b_need_update = false;
}

void CUICarBodyWnd::SendMessage(CUIWindow *pWnd, s16 msg, void *pData)
{

	if(msg == DRAG_DROP_ITEM_DRAG)
	{
		if (m_pCurrentDragDropItem) m_pCurrentDragDropItem->Highlight(false);
		PIItem pInvItem = (PIItem)((CUIDragDropItem*)pWnd)->GetData();
		m_pCurrentDragDropItem = (CUIDragDropItem*)pWnd;
		SetCurrentItem(pInvItem);
		m_pCurrentDragDropItem->Rescale(1.0f, 1.0f);
	}
	else if(msg == DRAG_DROP_ITEM_DB_CLICK)
	{
		bool bTakeAll = pData && (*(int*)pData==111) ;
		if (!bTakeAll && m_pCurrentDragDropItem) m_pCurrentDragDropItem->Highlight(false);
		m_pCurrentDragDropItem = (CUIDragDropItem*)pWnd;
		
		if(!bTakeAll){
			PIItem pInvItem = (PIItem)((CUIDragDropItem*)pWnd)->GetData();
			SetCurrentItem(pInvItem);
		}

		if(m_pCurrentDragDropItem->GetParent() == &UIOurBagList){
			ToOthersBag();
		}else if(m_pCurrentDragDropItem->GetParent() == &UIOthersBagList){
			ToOurBag();
		}else
			R_ASSERT2(false, "wrong parent for car_body wnd");
	}
	else if (BUTTON_CLICKED == msg && &UITakeAll == pWnd)
	{
		TakeAll();
	}
	else if(msg == DRAG_DROP_ITEM_RBUTTON_CLICK)
	{
		PIItem pInvItem				= (PIItem)((CUIDragDropItem*)pWnd)->GetData();
				
		SetCurrentItem				(pInvItem);
		m_pCurrentDragDropItem		= (CUIDragDropItem*)pWnd;
		
		ActivatePropertiesBox		();
	}
	else if(pWnd == &UIPropertiesBox &&	msg == PROPERTY_CLICKED)
	{
		if(UIPropertiesBox.GetClickedItem())
		{
			switch(UIPropertiesBox.GetClickedItem()->GetValue())
			{
			case INVENTORY_EAT_ACTION:	// 
				EatItem();
				break;
/*			case INVENTORY_ATTACH_ADDON:{
					m_pItemToUpgrade = (PIItem)pData;
					AttachAddon();
				}break;
			case INVENTORY_DETACH_SCOPE_ADDON:
				DetachAddon(*(smart_cast<CWeapon*>(m_pCurrentItem))->GetScopeName());
				break;
			case INVENTORY_DETACH_SILENCER_ADDON:
				DetachAddon(*(smart_cast<CWeapon*>(m_pCurrentItem))->GetSilencerName());
				break;
			case INVENTORY_DETACH_GRENADE_LAUNCHER_ADDON:
				DetachAddon(*(smart_cast<CWeapon*>(m_pCurrentItem))->GetGrenadeLauncherName());
				break;
*/
			case INVENTORY_UNLOAD_MAGAZINE:
				(smart_cast<CWeaponMagazined*>(m_pCurrentItem))->UnloadMagazine();
				break;
			}
		}
	}



	CUIWindow::SendMessage			(pWnd, msg, pData);
}



//////////////////////////////////////////////////////////////////////////

void CUICarBodyWnd::Draw()
{
	inherited::Draw();
}

//////////////////////////////////////////////////////////////////////////

void CUICarBodyWnd::Update()
{
	if(m_b_need_update||m_pInv->ModifyFrame()==Device.dwFrame || m_pOthersInv->ModifyFrame()==Device.dwFrame)
		UpdateLists		();

	if(m_pOurObject->Position().distance_to(m_pOthersObject->Position()) > 3.0f){
			GetHolder()->StartStopMenu(this,true);
	}
/*
	//  drag&drop    
	for(u32 i = 0; i <m_vDragDropItems.size(); ++i) 
	{
		CInventoryItem* pItem = (CInventoryItem*)m_vDragDropItems[i]->GetData();
		if(pItem && !pItem->Useful())
		{
			m_vDragDropItems[i]->GetParent()->DetachChild(m_vDragDropItems[i]);
			m_vDragDropItems[i]->SetData(NULL);
			m_vDragDropItems[i]->SetCustomDraw(NULL);

			xr_delete(m_vDragDropItems[i]);
			m_vDragDropItems.erase(m_vDragDropItems.begin()+i);
		}
	}
*/
	inherited::Update();
}

//////////////////////////////////////////////////////////////////////////

void CUICarBodyWnd::Show() 
{ 
	inherited::Show			();
	SetCurrentItem			(NULL);
	UpdateWeight			(UIOurBagWnd);
}

//////////////////////////////////////////////////////////////////////////

//     
// ,     
//  CUIInventoryWnd  
//  GetInventory()
bool CUICarBodyWnd::OurBagProc(CUIDragDropItem* pItem, CUIDragDropList* pList)
{
	CUICarBodyWnd* this_car_body_wnd =  smart_cast<CUICarBodyWnd*>(pList->GetParent()->GetParent());
	R_ASSERT2(this_car_body_wnd, "wrong parent addressed as trade wnd");

	PIItem pIItem = (PIItem)(pItem->GetData());
	
	//  -  
	NET_Packet						P;
	this_car_body_wnd->m_pOthersObject->u_EventGen(P,GE_OWNERSHIP_REJECT,
										this_car_body_wnd->m_pOthersObject->ID());
	P.w_u16							   (u16(pIItem->object().ID()));
	this_car_body_wnd->m_pOthersObject->u_EventSend	(P);

	// -   
	this_car_body_wnd->m_pOurObject->u_EventGen(P,GE_OWNERSHIP_TAKE,
										this_car_body_wnd->m_pOurObject->ID());
	P.w_u16							(u16(pIItem->object().ID()));
	this_car_body_wnd->m_pOurObject->u_EventSend	(P);

	return true;
}

//////////////////////////////////////////////////////////////////////////

bool CUICarBodyWnd::OthersBagProc(CUIDragDropItem* pItem, CUIDragDropList* pList)
{
	CUICarBodyWnd* this_car_body_wnd =  smart_cast<CUICarBodyWnd*>(pList->GetParent()->GetParent());
	R_ASSERT2(this_car_body_wnd, "wrong parent addressed as trade wnd");

	if (smart_cast<CBaseMonster*>(this_car_body_wnd->m_pOthersObject)) return false;

	PIItem pIItem = (PIItem)(pItem->GetData());

	float otherInvWeight	= this_car_body_wnd->m_pOthersInv->CalcTotalWeight();
	float otherMaxWeight	= this_car_body_wnd->m_pOthersInv->GetMaxWeight();
	float itmWeight			= pIItem->Weight();
	if	 (otherInvWeight+itmWeight >=otherMaxWeight)return false;
	// -  
	NET_Packet P;
	this_car_body_wnd->m_pOurObject->u_EventGen(P,GE_OWNERSHIP_REJECT,
									 this_car_body_wnd->m_pOurObject->ID());
	P.w_u16							 (u16(pIItem->object().ID()));
	this_car_body_wnd->m_pOurObject->u_EventSend			(P);

	//  -   
	this_car_body_wnd->m_pOthersObject->u_EventGen(P,GE_OWNERSHIP_TAKE,
										this_car_body_wnd->m_pOthersObject->ID());
	P.w_u16								(u16(pIItem->object().ID()));
	this_car_body_wnd->m_pOthersObject->u_EventSend	(P);

	return true;
}

//////////////////////////////////////////////////////////////////////////

bool CUICarBodyWnd::ToOurBag()
{
//	CTimer T;
	bool b;
//	T.Start();
	b= OurBagProc(m_pCurrentDragDropItem, &UIOurBagList);
//	Msg("----CUICarBodyWnd::ToOurBag:OurBagProc(): %f",T.GetElapsed_sec());

	if(!b)return false;
		

	((CUIDragDropList*)m_pCurrentDragDropItem->GetParent())->
											   DetachChild(m_pCurrentDragDropItem);
	

//	T.Start();
//	Msg("----CUICarBodyWnd::ToOurBag:AttachChild()---BEGIN: %f",T.GetElapsed_sec());
	UIOurBagList.AttachChild(m_pCurrentDragDropItem);
//	Msg("----CUICarBodyWnd::ToOurBag:AttachChild()---END: %f",T.GetElapsed_sec());

	m_pMouseCapturer = NULL;
	UpdateWeight(UIOurBagWnd);

	return true;
}

//////////////////////////////////////////////////////////////////////////

bool CUICarBodyWnd::ToOthersBag()
{
	if(!OthersBagProc(m_pCurrentDragDropItem, 
				      &UIOthersBagList)) return false;

	((CUIDragDropList*)m_pCurrentDragDropItem->GetParent())->DetachChild(m_pCurrentDragDropItem);
	UIOthersBagList.AttachChild(m_pCurrentDragDropItem);

	m_pMouseCapturer = NULL;
	UpdateWeight(UIOurBagWnd);

	return true;
}

//////////////////////////////////////////////////////////////////////////

void CUICarBodyWnd::DisableAll()
{
	UIOurBagWnd.Enable(false);
	UIOthersBagWnd.Enable(false);
}

//////////////////////////////////////////////////////////////////////////

void CUICarBodyWnd::EnableAll()
{
	UIOurBagWnd.Enable(true);
	UIOthersBagWnd.Enable(true);
}

void CUICarBodyWnd::SetCurrentItem(CInventoryItem* pItem)
{
	m_pCurrentItem = pItem;

	UIItemInfo.InitItem(m_pCurrentItem);
}

void CUICarBodyWnd::TakeAll()
{
	for (DRAG_DROP_LIST_it it = UIOthersBagList.GetDragDropItemsList().begin();
		 it != UIOthersBagList.GetDragDropItemsList().end();)
	{
		int iData = 111;
		SendMessage(*it++, DRAG_DROP_ITEM_DB_CLICK, (void*)(&iData) );
	}
	SetCurrentItem(NULL);
}


#include "../xr_level_controller.h"

bool CUICarBodyWnd::OnKeyboard(int dik, EUIMessages keyboard_action)
{
	if( inherited::OnKeyboard(dik,keyboard_action) )return true;

	if(keyboard_action==WINDOW_KEY_PRESSED&&key_binding[dik]==kUSE) {
			GetHolder()->StartStopMenu(this,true);
			return true;
	}
	return false;
}

void CUICarBodyWnd::ActivatePropertiesBox()
{
	float x,y;

	Frect rect = GetAbsoluteRect();
	GetUICursor()->GetPos(x,y);
		
	UIPropertiesBox.RemoveAll();
	
	CWeaponMagazined*		pWeapon			= smart_cast<CWeaponMagazined*>(m_pCurrentItem);
	CEatableItem*			pEatableItem	= smart_cast<CEatableItem*>(m_pCurrentItem);
    bool					b_show			= false;
	if(pWeapon && pWeapon->GetAmmoElapsed())
	{
			UIPropertiesBox.AddItem("Unload magazine",  NULL, INVENTORY_UNLOAD_MAGAZINE);
			b_show			= true;
	}
	
	
	if(pEatableItem)
	{
		UIPropertiesBox.AddItem("Eat",  NULL, INVENTORY_EAT_ACTION);
		b_show			= true;
	}

	if(b_show){
		UIPropertiesBox.AutoUpdateSize	();
		UIPropertiesBox.BringAllToTop	();
		UIPropertiesBox.Show			(x-rect.left, y-rect.top);
	}
}

void CUICarBodyWnd::EatItem()
{
	CActor *pActor = smart_cast<CActor*>(Level().CurrentEntity());
	if(!pActor) return;

	NET_Packet	P;
	CGameObject::u_EventGen(P, GEG_PLAYER_ITEM_EAT, Actor()->ID());
	P.w_u16		(m_pCurrentItem->object().ID());
	CGameObject::u_EventSend(P);

}
